/*
 * Copyright 2005-2007 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/*=============================================================================

    Module Name:            mfw_mxc_pf.c

    General Description:    Implementation of MPEG4 Deblocking on the IPU

===============================================================================
Portability:    This code is ported for Linux


===============================================================================
                            INCLUDE FILES
=============================================================================*/
#include <sys/ioctl.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <gst/gst.h>
#include <string.h>
#include "mxc_pf.h"
#include "mpeg4_dec_api.h"
#include "mfw_gst_mpeg4decoder.h"
/*=============================================================================
                            LOCAL FUNCTIONS
=============================================================================*/

/*=============================================================================
FUNCTION:		pf_init

DESCRIPTION:	pf_init() is used to initialise the deblocking 
                it allocates the memory required for input buffer,output
                buffer and the buffer required for QP values used in deblocking
                
ARGUMENTS PASSED:
        width       -   Width of the image to be deblocked.
		height		-	Height of the image to be deblocked.
		stride	    -	number of strides in the image.
        mpeg4_dec   -   mpeg4 decoder plugin context
		
RETURN VALUE:
				returns the status of the initialisaion.
			    0 - deblocking initialisation succeded
               -1.- error in initialisation

PRE-CONDITIONS:
        None

POST-CONDITIONS:
        None

IMPORTANT NOTES:
        None
=============================================================================*/
int pf_init(guint width, guint height, guint stride,MFW_GST_MPEG4_DECODER_INFO_T *mpeg4_dec)
{
        int i;
        int retval = 0;
        pf_init_params pf_init;
        pf_reqbufs_params pf_reqbufs;
        PFHandle *pfhandle;
        
        pfhandle = &mpeg4_dec->pf_handle;
        
        if ((pfhandle->fd_pf = open("/dev/mxc_ipu_pf", O_RDWR, 0)) < 0)
        {
                printf("Unable to open pf device\n");
                retval = -1;
                return retval;
        }

        memset(&pf_init, 0, sizeof(pf_init));
        pf_init.pf_mode = 3;
        pf_init.width = width;
        pf_init.height = height;
        pf_init.stride = stride;
        if (ioctl(pfhandle->fd_pf, PF_IOCTL_INIT, &pf_init) < 0)
        {
                printf("PF_IOCTL_INIT failed\n");
                retval = -1;
                close(pfhandle->fd_pf);
                return retval;
        }
        pfhandle->pf_qp_buf = mmap(NULL, pf_init.qp_size, 
                         PROT_READ | PROT_WRITE, MAP_SHARED, 
                         pfhandle->fd_pf, pf_init.qp_paddr);
        if (pfhandle->pf_qp_buf == NULL) {
                printf("v4l2_out test: mmap for qp buffer failed\n");
                retval = -1;
                close(pfhandle->fd_pf);
                return retval;
        }
        
        pf_reqbufs.count = 2;
        pf_reqbufs.req_size = 0;
        if (ioctl(pfhandle->fd_pf, PF_IOCTL_REQBUFS, &pf_reqbufs) < 0)
        {
                printf("PF_IOCTL_REQBUFS failed\n");
                retval = -1;
                close(pfhandle->fd_pf);
                return retval;
        }

        for (i = 0; i < pf_reqbufs.count; i++) {

                pfhandle->in_buf[i].index = i;
                if (ioctl(pfhandle->fd_pf, PF_IOCTL_QUERYBUF, &pfhandle->in_buf[i]) < 0)
                {
                        printf("PF_IOCTL_QUERYBUF failed\n");
                        retval = -1;
                        close(pfhandle->fd_pf);
                        return retval;
                }
                        
                pfhandle->pf_inbuf[i] = mmap (NULL, pfhandle->in_buf[i].size, 
                                 PROT_READ | PROT_WRITE, MAP_SHARED, 
                                 pfhandle->fd_pf, pfhandle->in_buf[i].offset);
                if (pfhandle->pf_inbuf[i] == NULL) {
                        printf("v4l2_out test: mmap for input buffer failed\n");
                        retval = -1;
                        close(pfhandle->fd_pf);
                        return retval;
                }
        }
        printf("PF driver initialized\n");
        return retval;
}


/*=============================================================================
FUNCTION:		pf_uninit

DESCRIPTION:	pf_uninit() is used to deallocate all the resources allocated for
                deblocking 
                
ARGUMENTS PASSED:
                mpeg4_dec   -   mpeg4 decoder plugin context

		
RETURN VALUE:
				returns the status of deallocation.
			    0 - deblocking deallocation succeded
               -1.- error in deallocation

PRE-CONDITIONS:
        None

POST-CONDITIONS:
        None

IMPORTANT NOTES:
        None
=============================================================================*/

int pf_uninit(MFW_GST_MPEG4_DECODER_INFO_T *mpeg4_dec)
{
        pf_reqbufs_params pf_reqbufs;
        int retval = 0;
        PFHandle *pfhandle;
        pfhandle = &mpeg4_dec->pf_handle;
        printf("Closing PF driver");
        
        /* deallocates the buffers allocated for PF*/

        pf_reqbufs.count = 0;   
        if (ioctl(pfhandle->fd_pf, PF_IOCTL_REQBUFS, &pf_reqbufs) < 0)
        {
                printf("PF_IOCTL_REQBUFS failed\n");
                retval = -1;
                return retval;
        }

        if (ioctl(pfhandle->fd_pf, PF_IOCTL_UNINIT, NULL) < 0)
        {
                printf("PF uninit failed\n");
                retval = -1;
        }

        munmap(pfhandle->pf_inbuf[0], pfhandle->in_buf[0].size);
        munmap(pfhandle->pf_inbuf[1], pfhandle->in_buf[1].size);
        close(pfhandle->fd_pf);
        printf(" - Done\n");
        return retval;
}        

/*=============================================================================
FUNCTION:		pf_start

DESCRIPTION:	pf_start() activates the deblocking process.
                
                
ARGUMENTS PASSED:
                in_buf_idx  -   index of the input buffer
                out_buff    -   offset of the outbut buffer
                mpeg4_dec   -   mpeg4 decoder plugin context

		
RETURN VALUE:
				returns the status of deblocking.
			    0 - deblocking succeded
               -1.- error in deblocking

PRE-CONDITIONS:
        None

POST-CONDITIONS:
        None

IMPORTANT NOTES:
        None
=============================================================================*/

int pf_start(int in_buf_idx, unsigned long out_buf,MFW_GST_MPEG4_DECODER_INFO_T *mpeg4_dec)
{
        int retval = 0;
        pf_start_params pf_st;
        PFHandle *pfhandle;

        pfhandle = &mpeg4_dec->pf_handle;
        memset(&pf_st, 0, sizeof(pf_st));
        pf_st.wait = 0;
        pf_st.in = pfhandle->in_buf[in_buf_idx];
        pf_st.out.index = -1;
        pf_st.out.size = 0;    
        pf_st.out.offset = out_buf;

        /*g_print(" before PF_IOCTL_START\n");*/
        if ((retval = ioctl(pfhandle->fd_pf, PF_IOCTL_START, &pf_st)) < 0)
        {
                printf("PF start failed: %d\n", retval);
                return retval;
        }

        /*g_print(" after PF_IOCTL_START\n");*/
        return retval;
}

/*=============================================================================
FUNCTION:		pf_wait

DESCRIPTION:	pf_wait()  puts the processor in wait state untill 
                deblocking is completed.
                
                
ARGUMENTS PASSED:
                mpeg4_dec   -   mpeg4 decoder plugin context
		
RETURN VALUE:
				returns the status of wait during deblocking.
			    0 - wait for deblocking succeded
               -1.- error in wait for deblocking

PRE-CONDITIONS:
        None

POST-CONDITIONS:
        None

IMPORTANT NOTES:
        None
=============================================================================*/

int pf_wait(MFW_GST_MPEG4_DECODER_INFO_T *mpeg4_dec)
{
        int retval = 0;

        if ((retval = ioctl(mpeg4_dec->pf_handle.fd_pf, PF_IOCTL_WAIT, NULL)) < 0)
        {
                printf("PF wait failed: %d\n", retval);
        }
        return retval;
}
